# -*- coding: utf-8 -*-
"""
Created on Wed Mar 26 ‏‎12:28:44 2014

@author: Daniel Treceño

det_AD -> activity-rest

It reads the input channel and returns a mask of activity-rest 

Input:
------
channel -> the current channel in the application

Output:
-------
filtro -> the activity mask where the rest is 0 and the a normal activity is 1

Algorithm:
----------
Referencia:

D. Martín, P. Casaseca, S. Alberola, J.A. López, F.C. Ruiz, J.M. Andrés, J.R. Garmendia and J.
Ardura. Automatic Diagnosis of ADHD Based on Nonlinear Analysis of Actimetry Registries. 
In 36th International Conference on Acoustics, Speech and Signal Processing (ICASSP'2011), 2011.
"""

import numpy

def det_AD(channel):
    s = numpy.asarray(channel,dtype=float)
    s = s/s.max()
    
    M = 1*1800 #max size of 30 minutes
    h = numpy.ones((1,M),dtype=float)
        
    #triangular
    lime = 1;
    limi = 0;
    mascara = numpy.concatenate((numpy.linspace(lime, limi, M/2), numpy.linspace(limi, lime, M/2)))
        
    h = numpy.multiply(h,mascara)
    h = h/numpy.sum(h)
    h = numpy.squeeze(numpy.asarray(h))
    
    w = numpy.convolve(h, s)
    desfase = M/2
    w = w[desfase-1:-desfase]
    #w = numpy.take(w,range(desfase-1,w.__len__()-desfase));
    w = w/numpy.max(w)
    
    th = 0.025
    k = 1
    
    while k:
        #g = g + 1
        filtro = w > th      
        s = numpy.multiply(s,filtro)      
        w = numpy.convolve(h, s)
        w = w[desfase-1:-desfase]
        w = w/numpy.max(w);
        filtro2 = w > th     
        if numpy.sum(numpy.abs(filtro - filtro2))<= 60:
            #print g
            k = 0;
        filtro = filtro2;
        
    Gf = numpy.abs(numpy.diff(filtro));  
    Gf = Gf.astype(bool) 
    Posiciones = numpy.transpose(range(2,filtro.__len__()+1)) 
    Posiciones = numpy.multiply(Posiciones,Gf);
    Posiciones = Posiciones[Posiciones>0]
    
    for k in range(0,Posiciones.__len__()-1):
        if (Posiciones[k+1] - Posiciones[k]) <= 1800:
            filtro[Posiciones[k]-1:Posiciones[k+1]] = numpy.multiply(filtro[Posiciones[k]-2],numpy.squeeze(numpy.asarray(numpy.ones((1,Posiciones[k+1]-Posiciones[k]+1),dtype=float))))
            #print(filtro[Posiciones[k]-1:Posiciones[k+1]].__len__())
    
    filtro = filtro.astype(bool)
    it = 1;
    while it:
        Pos  = range(0,filtro.__len__())
        Pos = numpy.asarray(Pos)
        PosD = Pos[filtro==0]
        mD = numpy.median(s[filtro==0])
        mA = numpy.median(s[filtro])
            
        it = 0
        if numpy.max(PosD) < s.__len__()-1:
            if (numpy.abs(s[numpy.max(PosD) + 1] - mD) < numpy.abs(s[numpy.max(PosD) + 1] - mA)):
                filtro[numpy.max(PosD)+1] = 0
                it = 1
        if numpy.min(PosD) > 0:
            if (numpy.abs(s[numpy.min(PosD) - 1] - mD) < numpy.abs(s[numpy.min(PosD) - 1] - mA)):
                filtro[numpy.min(PosD) - 1] = 0
                it = 1;
    
    cadena = filtro.astype(int).tolist().__str__()
    cadena = cadena.replace(",","")
    cadena = cadena.replace(" ","")
    cadena = cadena.replace("[","")
    cadena = cadena.replace("]","")
    
    SleepOnset = []
    SleepEnd = []
    for i in range(0,cadena.__len__()-1):
        c = cadena[i:i+2]
        if c == '10':
            SleepOnset.append(i)
        elif c == '01':
            SleepEnd.append(i)
        else:
            pass
    
    if min(SleepEnd) < min(SleepOnset):    
        SleepEnd = SleepEnd[1:]
    
        
    if max(SleepOnset) > max(SleepEnd):
        SleepOnset = SleepOnset[0:-1]
        
    SleepOnset = numpy.asarray(SleepOnset)
    SleepEnd = numpy.asarray(SleepEnd)
    Duraciones = SleepEnd - SleepOnset
    
    InisCorrectos = SleepOnset[Duraciones>1800]
    FinesCorrectos = SleepEnd[Duraciones>1800]
    
    filtro2 = numpy.ones((1,filtro.__len__()),dtype=int);
    filtro2 = numpy.squeeze(filtro2)
    
    for k in range(0,InisCorrectos.__len__()):
        filtro2[InisCorrectos[k]:FinesCorrectos[k]+1] = numpy.zeros(FinesCorrectos[k] - InisCorrectos[k]+1,dtype=int)
    
    filtro = filtro2 
    Output = {'Filter':filtro.astype(int).tolist(),
        'A_R_Start':InisCorrectos.tolist(),
        'A_R_End':FinesCorrectos.tolist()}
    return Output
    