#encoding:utf-8

#import sessions
from django.contrib.sessions.models import Session

#import models
from app.models import file_data, file_header, file_channel, file_mask, session_data, session_channel_mask

#import program
from download_results.program.general_def import apply_changes_general
from download_results.program.statistical_analysis_def  import compute_statistic
from download_results.program.sleep_analysis_def import sleep_analysis_def
from download_results.program.cosinor_def import EstimarCosinor
from download_results.program.symbolic_dynamic_def import DinSym
from download_results.program.ctm_def import ctm_fun
from download_results.program.KNN.KNN_class import knn_class_run as class_knn

#django render web
from django.shortcuts import render_to_response
from django.template import RequestContext

#cache decorators
from django.views.decorators.cache import never_cache

#login
from django.contrib.auth.decorators import login_required

#excel
import xlwt
from xlrd import open_workbook
from xlutils.copy import copy

#JSON
from django.utils import simplejson

#send file
from django.http import HttpResponse
from django.core.servers.basehttp import FileWrapper

#settings
from django.conf import settings

#numpy
from  numpy import fromstring, linspace

#time
from datetime import datetime

#OS
import os

#String
import string


# Download results
"""
Esta página debe seguir varios pasos.
	-Elegir el archivo que se desea analizar (dar a "Next")
		-Si no hay ningún archivo el botón "Next" estará inhabilitado
		-Si hay archivos, estos se mostrarán mediante un select desplegable
	-Rellenar el formulario
		-Hasta que todo el formulario no esté bien, se inhabilitará el botón "Next"
		-A la web se le pasará un dict con todo lo necesario para corregir el formularo usando JS
	-Una vez relleno el form correctamente se habilitará el botón "Continue"
	-Aparecerá una animación de carga
	-se retornará un enlace para descargar el archivo
"""
@never_cache
@login_required(login_url='/log_in')
def download_results(request):	
	if request.method=='POST':
		if 'Go_form' in request.POST: #botton name
			var = 2
			ID = request.POST['select_data']
			current_file_dat = file_data.objects.get(pk=ID)
			#select channels
			select_channels = []
			i = 0
			for p in str_to_list_str(current_file_dat.channels_name):
				select_channels = select_channels  + [(i,p)]
				i = i+1
			#minimun decimate
			epoch = str_to_list(current_file_dat.epoch_period)
			decim = epoch[0]*3600+epoch[1]*60+epoch[2]
			#activity-rest, start and end
			start_time = str_to_list(current_file_dat.start_time)
			start_time_s = start_time[0]*3600*24+start_time[1]*3600+start_time[2]*60+start_time[3]
			ar_s_s = str_to_list(current_file_dat.a_r_start)[0] + start_time_s
			ar_e_s = str_to_list(current_file_dat.a_r_end)[0] + start_time_s
			ar_start = s_to_dhms(ar_s_s)
			ar_end = s_to_dhms(ar_e_s)

			dict_web={'user':request.user,
					'select_channels':select_channels,
					'decim':decim,
					'start_time':start_time,
					'interval_end':str_to_list(current_file_dat.interval_end),
					'ar_start':ar_start,
					'ar_end':ar_end,
					'ID': ID,
					'var':var,}
	else:
		var = 1
		options = file_data.objects.filter(user_name=request.user)
		dict_web={'user':request.user,
				'var':var,
				'options':options,}
	return render_to_response('download_results/download_results.html',dict_web, context_instance=RequestContext(request))

@never_cache
@login_required(login_url='/log_in')
def generate_results(request):
	#Read data
	#---------
	ID_file = request.GET['ID_file']
	current_file_light = file_data.objects.get(pk=ID_file)
	descriptive_name = current_file_light.descriptive_name
	notes = current_file_light.notes
	channel_id = int(request.GET['channel'])
	b = file_channel.objects.filter(regarding=current_file_light).get(number=channel_id)
	c = file_mask.objects.get(regarding=current_file_light)
	channel = str_to_list_float(b.channel)
	mask = str_to_list_float(c.mask)
	start_time = str_to_list(request.GET['start_time'])
	#General
	decimate = int(request.GET['decimate'])
	interval_onset = str_to_list(request.GET['interval_onset'])
	interval_end = str_to_list(request.GET['interval_end'])
	recalculate_ad = int(request.GET['recalculate_ad'])
	if recalculate_ad == 1:
		recal_str = "True"
	else:
		recal_str = "False"
	#statistical analysis
	stat_frame = int(request.GET['stat_frame'])
	#sleep analysis
	sleep_mode = int(request.GET['sleep_mode'])
	sleep_onset = str_to_list(request.GET['sleep_onset'])
	sleep_end = str_to_list(request.GET['sleep_end'])
	#cosinor analysis
	rhythm = str_to_list(request.GET['rhythm'])
	alpha = float(request.GET['alpha'])
	#symbolic dynamics
	symbolic_frame = int(request.GET['symbolic_frame'])
	alphabet = int(request.GET['alphabet'])
	symbols = int(request.GET['symbols'])
	overlaping = int(request.GET['overlaping'])
	#ctm
	ctm_frame = int(request.GET['ctm_frame'])
	rho = int(request.GET['rho'])
	#classifier
	c_type = int(request.GET['c_type'])
	k = int(request.GET['k'])
	t1c = str_to_list(request.GET['t1c'])
	t2c = str_to_list(request.GET['t2c'])
	#GENERAL
	#-------
	current_data, current_mask = apply_changes_general(decimate, interval_onset, interval_end, start_time, recalculate_ad, channel, mask)
	#STATISTICAL ANALYSIS
	#--------------------
	stat_data = compute_statistic(channel,mask,stat_frame)
	#SLEEP ANALYSIS
	#--------------
	sleep_data = sleep_analysis_def (channel, sleep_onset, sleep_end, start_time)
	#COSINOR ANALYSIS
	#----------------
	alpha = alpha/100.
	r = float(rhythm[0]*24*3600+rhythm[1]*3600+rhythm[2]*60+rhythm[3])/24./60./60.
	onset = float(interval_onset[0]*3600*24+interval_onset[1]*3600+interval_onset[2]*60+interval_onset[3])/24./60./60.
	end = float(interval_end[0]*3600*24+interval_end[1]*3600+interval_end[2]*60+interval_end[3])/24./60./60.
	tC = linspace(onset,end,current_data.__len__())
	cosinor_data = EstimarCosinor(current_data, tC, r, alpha)
	#print 'mesor', cosinor_data['M'], '0',cosinor_data['CI_M']
	#print 'Amplitude', cosinor_data['A'], cosinor_data['CI_A'][0], cosinor_data['CI_A'][1]
	#print 'Acrophase', cosinor_data['Acrophase'][0], cosinor_data['Acrophase'][1], cosinor_data['Acrophase'][2]
	#print 'p-value', cosinor_data['pvalor'], '0', '0'
	#SYMBOLIC_DYNAMICS
	#-----------------
	#print symbolic_frame, alphabet, symbols, overlaping
	symbolic_data = DinSym(current_data,current_mask,symbolic_frame, alphabet, symbols, overlaping, 1)
	#CTM
	#---
	ctm_data = ctm_fun(current_data,current_mask,symbolic_frame,rho)
	#CLASSIFIER
	#----------
	if c_type == 1:
		type_class = 'KNN'
		names = str_to_list_str(current_file_light.channels_name)
		try:
			ind_z = int(names.index(' z Axis'))
		except:
			ind_z = -1
		try:
			ind_r = int(names.index(' Vector Module'))
		except:
			ind_r = -1
		if (ind_r == -1 | ind_z == -1):
			results = 'z Axis and Vector Module are necessary'
			out = {'result':'z Axis and Vector Module are necessary',
				'datum_x':'',
				'datum_y':''}
		else:
			print t1c, t2c
			channelc = str_to_list_float(file_channel.objects.filter(regarding=current_file_light).get(number=ind_z).channel)
			maskc = str_to_list_float(file_mask.objects.get(regarding=current_file_light).mask)
			c1_data, c1_mask = apply_changes_general(1, t1c, t2c, start_time, 0, channelc, maskc)
			c1 = ctm_fun(c1_data, c1_mask,3, 45)['CTM']
			channelc = str_to_list_float(file_channel.objects.filter(regarding=current_file_light).get(number=ind_r).channel)
			c2_data, c2_mask = apply_changes_general(30, t1c, t2c, start_time, 0, channelc, maskc)
			c2 = ctm_fun(c2_data, c2_mask,1, 22)['CTM']
			p = [c1,c2]
			data_class = class_knn(p,k)
	#EXCEL
	#-----
	header_style = xlwt.easyxf('font: name Times New Roman, colour green, bold on')
	date_style = xlwt.easyxf('',num_format_str='DD/MM/YY HH:MM AM/PM')
	excel = xlwt.Workbook()
	#excel_name = remove_accents(str(request.user)) + '_' + remove_accents(descriptive_name) + '_' + str(datetime.today()) + '.xls'
	date_now = datetime.now()
	excel_name = str(request.user) + '_' + str(date_now.year) + '_' + str(date_now.month) + '_' + str(date_now.day) + '_' + str(date_now.hour)  + '_' + str(date_now.minute)  + '_' + str(date_now.second) + '.xls'
	#print datetime.now(), type(datetime.now())
	#general
	general = excel.add_sheet('General',cell_overwrite_ok=True)
	general.write(0, 0, 'Descriptive name', header_style)
	general.write(0, 1, descriptive_name)
	general.write(1, 0, 'Date of creation', header_style)
	general.write(1, 1, datetime.now(), date_style)
	general.write(2, 0, 'Original file name', header_style)
	general.write(2, 1, current_file_light.file_name)
	general.write(3, 0, 'Decimate', header_style)
	general.write(3, 1, decimate)
	general.write(4, 0, 'Channel', header_style)
	general.write(4, 1, str_to_list_str(current_file_light.channels_name)[channel_id])
	general.write(5,0, 'Original interval', header_style)
	general.write(6,1, 'Onset', header_style)
	general.write(6,2, dhms_to_str(start_time) )
	general.write(7,1, 'End', header_style)
	general.write(7,2, dhms_to_str(str_to_list(current_file_light.interval_end)))
	general.write(8,0, 'Current interval', header_style)
	general.write(9,1, 'Onset', header_style)
	general.write(9,2, dhms_to_str(interval_onset))
	general.write(10,1, 'End', header_style)
	general.write(10,2, dhms_to_str(interval_end))
	general.write(11,0, 'Recalculate A-D Mask',header_style)
	general.write(11,1, recal_str)
	general.write(12,0, 'Notes', header_style)
	general.write(13,0, str(current_file_light.notes))
	#statistical analysis
	stat = excel.add_sheet('Statistical Analysis',cell_overwrite_ok=True)
	stat.write(0, 0, 'Options', header_style)
	stat.write(1, 1, 'frame', header_style)
	stat.write(1, 2, name_frame(stat_frame))
	stat.write(2, 0, 'Results', header_style)
	stat.write(3, 1, 'mean', header_style)
	stat.write(3, 2, stat_data['mean'])
	stat.write(4, 1, 'median', header_style)
	stat.write(4, 2, stat_data['std'])
	stat.write(5, 1, 'std', header_style)
	stat.write(5, 2, stat_data['mean'])
	stat.write(6, 1, 'iqr', header_style)
	stat.write(6, 2, stat_data['iqr'])
	stat.write(7, 1, 'skew', header_style)
	stat.write(7, 2, stat_data['skew'])
	stat.write(8, 1, 'kurtosis', header_style)
	stat.write(8, 2, stat_data['kurtosis'])
	stat.write(9, 1, 'vc', header_style)
	stat.write(9, 2, stat_data['vc'])
	#sleep analysis
	sleep = excel.add_sheet('Sleep Analysis',cell_overwrite_ok=True)
	sleep.write(0, 0, 'Options', header_style)
	sleep.write(1, 1, 'Sleep Onset', header_style)
	sleep.write(1, 2, dhms_to_str(sleep_onset))
	sleep.write(2, 1, 'Sleep End', header_style)
	sleep.write(2, 2, dhms_to_str(sleep_end))
	sleep.write(3, 0, 'Results', header_style)
	sleep.write(4, 1, 'algorithm', header_style)
	sleep.write(4, 2, sleep_data['algorithm'])
	sleep.write(5, 1, 'SleepOnset', header_style)
	sleep.write(5, 2, sleep_data['SleepOnset'])
	sleep.write(6, 1, 'TotalSleepTime', header_style)
	sleep.write(6, 2, sleep_data['TotalSleepTime'])
	sleep.write(7, 1, 'WakeAfterSleepOnset', header_style)
	sleep.write(7, 2, sleep_data['WakeAfterSleepOnset'])
	sleep.write(8, 1, 'Awakenings', header_style)
	sleep.write(8, 2, sleep_data['Awakenings'])
	sleep.write(9, 1, 'AverageAwakening', header_style)
	sleep.write(9, 2, sleep_data['AverageAwakening'])
	sleep.write(10, 1, 'TotalCounts', header_style)
	sleep.write(10, 2, sleep_data['TotalCounts'])
	sleep.write(11, 1, 'Efficiency', header_style)
	sleep.write(11, 2, sleep_data['Efficiency'])
	sleep.write(12, 1, 'LSInterval', header_style)
	sleep.write(12, 2, sleep_data['LSInterval'])
	sleep.write(13, 1, 'PQSleep', header_style)
	sleep.write(13, 2, sleep_data['PQSleep'])
	#Cosinor analysis
	cosinor = excel.add_sheet('Cosinor Analysis',cell_overwrite_ok=True)
	cosinor.write(0, 0, 'Options', header_style)
	cosinor.write(1, 1, 'Alpha', header_style)
	cosinor.write(1, 2, str(int(alpha*100)))
	cosinor.write(2, 1, 'Rhythm', header_style)
	cosinor.write(2, 2, dhms_to_str(rhythm))
	cosinor.write(3, 0, 'Results', header_style)
	cosinor.write(4, 2, 'Estimated Value', header_style)
	cosinor.write(4, 3, 'CI: Inferior Limit', header_style)
	cosinor.write(4, 4, 'CI: Superior Limit', header_style)
	cosinor.write(5, 1, 'Mesor', header_style)
	cosinor.write(5, 2, str(cosinor_data['M']))
	cosinor.write(5, 3, '0')
	cosinor.write(5, 4, str(cosinor_data['CI_M']))
	cosinor.write(6, 1, 'Amplitude', header_style)
	cosinor.write(6, 2, str(cosinor_data['A']))
	cosinor.write(6, 3, str(cosinor_data['CI_A'][0]))
	cosinor.write(6, 4, str(cosinor_data['CI_A'][1]))
	cosinor.write(7, 1, 'Acrophase', header_style)
	cosinor.write(7, 2, str(cosinor_data['Acrophase'][0]))
	cosinor.write(7, 3, str(cosinor_data['Acrophase'][1]))
	cosinor.write(7, 4, str(cosinor_data['Acrophase'][2]))
	cosinor.write(8, 1, 'p-value', header_style)
	cosinor.write(8, 2, str(cosinor_data['pvalor']))
	cosinor.write(8, 3, '0')
	cosinor.write(8, 4, '0')
	#Symbolic Dynamics
	symbolic = excel.add_sheet('Symbolic Dynamics',cell_overwrite_ok=True)
	symbolic.write(0, 0, 'Options', header_style)
	symbolic.write(1, 1, 'frame', header_style)
	symbolic.write(1, 2, name_frame(symbolic_frame))
	symbolic.write(2, 1, 'alphabet', header_style)
	symbolic.write(2, 2, str(alphabet))
	symbolic.write(3, 1, 'symbols', header_style)
	symbolic.write(3, 2, str(symbols))
	symbolic.write(4, 1, 'overlaping', header_style)
	symbolic.write(4, 2, str(overlaping))
	symbolic.write(5, 0, 'Results', header_style)
	symbolic.write(6, 1, 'Word', header_style)
	symbolic.write(6, 2, 'Probability', header_style)
	i = 7
	for p in symbolic_data['dr']:
		symbolic.write(i, 1, str(p[0]))
		symbolic.write(i, 2, str(p[1]))
		i = i + 1
	#Ctm
	ctm = excel.add_sheet('CTM',cell_overwrite_ok=True)
	ctm.write(0, 0, 'Options', header_style)
	ctm.write(1, 1, 'frame', header_style)
	ctm.write(1, 2, name_frame(ctm_frame))
	ctm.write(2, 1, 'rho', header_style)
	ctm.write(2, 2, str(rho))
	ctm.write(3, 0, 'Results', header_style)
	ctm.write(4, 1, 'ctm', header_style)
	ctm.write(4, 2, str(ctm_data['CTM']))
	#Classifier
	classifier = excel.add_sheet('Classifier',cell_overwrite_ok=True)
	classifier.write(0, 0, 'Options', header_style)
	classifier.write(1, 0, 'Classifier', header_style)
	classifier.write(1, 1, type_class)
	classifier.write(2,0, 'Interval', header_style)
	classifier.write(3,1, 'Onset', header_style)
	classifier.write(3,2, dhms_to_str(t1c))
	classifier.write(4,1, 'End', header_style)
	classifier.write(4,2, dhms_to_str(t2c))
	classifier.write(5, 1, 'K', header_style)
	classifier.write(5, 2, k)
	classifier.write(6, 0, 'Results', header_style)
	classifier.write(7, 1, 'Feature 1', header_style)
	classifier.write(7, 2, data_class['datum_x'])
	classifier.write(8, 1, 'Feature 2', header_style)
	classifier.write(8, 2, data_class['datum_y'])
	classifier.write(9, 1, 'Result', header_style)
	classifier.write(9, 2, data_class['result'])
	#save excel
	excel.save(settings.MEDIA_ROOT+"/"+excel_name)
	#output -> excel name
	out = {'name':excel_name}
	return HttpResponse(simplejson.dumps(out), mimetype='application/json')

@never_cache
@login_required(login_url='/log_in')
def get_results(request):
	File_name = request.GET['name']
	user = str(request.user)
	if user == File_name[0:len(user)]:
		path = str(settings.MEDIA_ROOT+"/"+File_name)
		try:
			rd = open_workbook(path,formatting_info=True,on_demand=True)
		except:
			rd = -1
		if rd == -1:
			response = HttpResponse('The file has already been downloaded')
		else:
			excel = copy(rd)
			os.remove(path)
			#response = HttpResponse(mimetype="application/ms-excel")
			response = HttpResponse(mimetype="application/vnd.ms-excel")
			response['Content-Disposition'] = 'attachment; filename='+File_name
			#response['X-Sendfile'] = settings.MEDIA_ROOT+"/"+File_name
			excel.save(response)
	else:
		response = HttpResponse('invalid URL')
	return response

#seconds to [dd,hh,mm,ss]
def s_to_dhms (s):
    #s -> seconds
    time = [0,0,0,0]
    time[3] = int(s%60) #seconds
    mins = int(s/60)
    time[2] = int(mins%60)
    h = int(mins/60)
    time[1] = int(h%24)
    time[0] = int(h/24)
    return time

def str_to_list(string):
	out = fromstring(string,dtype=int, sep=',')
	return out.tolist()

def str_to_list_float(string):
	out = fromstring(string,dtype=float, sep=',')
	return out.tolist()

def str_to_list_str(string):
	aux = ''
	out = []
	for i in range(len(string)):
		if string[i] == ',':
			out.append(aux)
			aux = ''
		elif i == len(string)-1:
			aux = aux + string[i]
			out.append(aux) 
		else:
			aux = aux + string[i]
	return out

def name_frame(num):
	if num==1:
		out = 'Activity'
	elif num==2:
		out = 'Rest'
	elif num==3:
		out = 'Whole'
	return out

def dhms_to_str(vector):
	day = 'day(' + str(vector[0]) + ') '
	hour = '0'+str(vector[1])
	hour = hour[-2::]
	minu = '0'+str(vector[2])
	minu = minu[-2::]
	sec = '0'+str(vector[3])
	sec = sec[-2::]
	return day + hour + ':' + minu + ':' + sec

def remove_accents(data):
    return filter(lambda char: char in string.ascii_lowercase, data.lower())
